/*
    Copyright 2009 Graham King <graham@gkgk.org>

    This file is part of Foxden.

    Foxden is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foxden is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foxden.  If not, see <http://www.gnu.org/licenses/>.
*/

var FDP = {
    
    start:
        function() {
            FDP._addl('fd_add', FDP.onAdd);
            FDP._addl('fd_edit', FDP.onEdit);
            FDP._addl('fd_delete', FDP.onDelete);

            FDP.updateList();
        },
        
    updateList:
        function() {

            var layoutList = document.getElementById('fd_layout_list');
            while (layoutList.itemCount != 0) {
                layoutList.removeItemAt(0);
            }

            var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                          .getService(Components.interfaces.nsIPrefService);
            var branch = prefs.getBranch("extensions.foxden.layout.");
            var prefTree = branch.getChildList("", {});
            
            prefTree.forEach(
                     function(prefName, index, arr) {
                        var prefParts = prefName.split('.');
                        if (prefParts[1] == 'name') {
                            var friendlyName = Util.getCharPref('layout.'+ prefName);
                            FDP._addLayout(prefParts[0], friendlyName);
                        }
                     }
            );

        },
        
    _addl:
        function(id, func) {
            var control = document.getElementById(id);
            control.addEventListener('command', func, false);
        },
        
    _addLayout:
        function(slug, name) {
            var entry = document.createElement('listitem');
            entry.setAttribute('label', name);
            entry.setAttribute('id', slug);
            document.getElementById('fd_layout_list').appendChild(entry);
        },

    onAdd:
        function() {
            FDP._openLayoutEditor(null);
        },

    onEdit:
        function() {
            var target = document.getElementById('fd_layout_list').selectedItem;
            if (! target) {
                alert('Please use the list to select a layout to edit');
                return;
            }
            
            var slug = target.getAttribute('id');
            FDP._openLayoutEditor(slug);
        },
        
    _openLayoutEditor:
        function(slug) {
            openDialog(
                        'chrome://foxden/content/config_layout.xul',
                        'Foxden layout editor',
                        '',
                        FDP.updateList,
                        slug
                        );            
        },
        
    onDelete:
        function() {
            var layoutList = document.getElementById('fd_layout_list');
            var target = layoutList.selectedItem;
            if (! target) {
                alert('Please use the list to select a layout to delete');
                return;
            }
            
            var slug = target.getAttribute('id');
            Util.clearPref('layout.'+ slug +'.name');
            Util.clearPref('layout.'+ slug +'.layout');
            
            layoutList.removeItemAt( layoutList.selectedIndex );
            
            FDP.updateList();
        }
}

var FDL = {
    
    updateFunc: null,
    slug: null,
    
    startLayoutEditor:
        function(updateFunc, slug) {
            
            FDL.updateFunc = updateFunc;
            
            if (slug) {
                FDL.slug = slug;
                                
                var name = Util.getCharPref('layout.'+ FDL.slug +'.name');
                var layout = Util.getCharPref('layout.'+ FDL.slug +'.layout');
                
                var nameControl = document.getElementById('fd_layout_name');
                var layoutControl = document.getElementById('fd_layout_input')
                
                nameControl.setAttribute('value', name);
                layoutControl.setAttribute('value', layout);
            }
        },
        
    saveLayout:
        function() {
            var name = document.getElementById('fd_layout_name').value;
            var layout = document.getElementById('fd_layout_input').value;
            
            // FDL.slug being set means we are editing an existing layout
            if ( ! FDL.slug ) {
                FDL.slug = Util.slugify(name);
            }
            
            Util.setCharPref('layout.'+ FDL.slug +'.name', name);
            Util.setCharPref('layout.'+ FDL.slug +'.layout', layout);
            
            FDL.updateFunc();
        }
}